/**
 * Licensed to the Apache Software Foundation (ASF) under one or more
 * contributor license agreements.  See the NOTICE file distributed with
 * this work for additional information regarding copyright ownership.
 * The ASF licenses this file to You under the Apache License, Version 2.0
 * (the "License"); you may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.apache.blur.manager.command.primitive;

import java.io.IOException;
import java.util.Iterator;
import java.util.Map;
import java.util.Map.Entry;

import org.apache.blur.manager.command.Args;
import org.apache.blur.manager.command.ClusterCommand;
import org.apache.blur.manager.command.CommandAggregator;
import org.apache.blur.manager.command.CommandContext;

@SuppressWarnings("serial")
public class DocumentCountAggregator extends DocumentCount implements ClusterCommand<Long>,
    CommandAggregator<Integer, Long> {

  private static final String DOC_COUNT_AGGREGATE = "docCountAggregate";

  @Override
  public String getName() {
    return DOC_COUNT_AGGREGATE;
  }

  @Override
  public Long aggregate(Iterator<Entry<String, Integer>> it) throws IOException {
    long total = 0;
    while (it.hasNext()) {
      total += it.next().getValue();
    }
    return total;
  }

  @Override
  public Long clusterExecute(Args args, CommandContext context) {
    // where the key is the server hostname
    Map<String, Long> results = context.execute(args, DOC_COUNT_AGGREGATE);
    long total = 0;
    for (Entry<String, Long> e : results.entrySet()) {
      total += e.getValue();
    }
    return total;
  }

}
